/*
 * Project            	GeometryKit
 * Class              	G3DFunctions
 * Creator            	Philippe C.D. Robert
 * Maintainer         	Philippe C.D. Robert
 * Creation Date      	Thu Sep  9 11:05:08 CEST 1999
 *
 * Copyright (c) Philippe C.D. Robert
 *
 * The GeometryKit is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU LGPL Version 2 as published by the Free 
 * Software Foundation
 *
 * A lot of the code found here is based on algorithms and code examples found
 * in the Graphics Gem books and papers from GameDeveloper!
 *
 * $Id: G3DFunctions.c,v 1.34 2001/01/05 20:50:47 robert Exp $
 *
 */

#include "G3DFunctions.h"
#include "G3DDefs.h"

/******************************************************************************
 *
 * Some usefuls vectors and matrices
 *
 *****************************************************************************/

float  G3DOrigin3f[3] = {0.0f,0.0f,0.0f};
double G3DOrigin3d[3] = {0.0,0.0,0.0};
float  G3DOrigin4f[4] = {0.0f,0.0f,0.0f,0.0f};
double G3DOrigin4d[4] = {0.0,0.0,0.0,0.0};

float  G3DUnityVecX2f[2] = {1.0f,0.0f};
float  G3DUnityVecY2f[2] = {0.0f,1.0f};
double  G3DUnityVecX2d[2] = {1.0 ,0.0 };
double  G3DUnityVecY2d[2] = {0.0 ,1.0 };

float  G3DUnityVecX3f[3] = {1.0f,0.0f,0.0f};
float  G3DUnityVecY3f[3] = {0.0f,1.0f,0.0f};
float  G3DUnityVecZ3f[3] = {0.0f,0.0f,1.0f};
double  G3DUnityVecX3d[3] = {1.0 ,0.0 ,0.0 };
double  G3DUnityVecY3d[3] = {0.0 ,1.0 ,0.0 };
double  G3DUnityVecZ3d[3] = {0.0 ,0.0 ,1.0 };

float  G3DUnityVecX4f[4] = {1.0f,0.0f,0.0f,0.0f};
float  G3DUnityVecY4f[4] = {0.0f,1.0f,0.0f,0.0f};
float  G3DUnityVecZ4f[4] = {0.0f,0.0f,1.0f,0.0f};
float  G3DUnityVecW4f[4] = {0.0f,0.0f,0.0f,1.0f};
double  G3DUnityVecX4d[4] = {1.0 ,0.0 ,0.0 ,0.0 };
double  G3DUnityVecY4d[4] = {0.0 ,1.0 ,0.0 ,0.0 };
double  G3DUnityVecZ4d[4] = {0.0 ,0.0 ,1.0 ,0.0 };
double  G3DUnityVecW4d[4] = {0.0 ,0.0 ,0.0 ,1.0 };

float  G3DIdentityMatrix3f[9] = {1.0f,0.0f,0.0f,0.0f,1.0f,0.0f,0.0f,0.0f,1.0f};
double G3DIdentityMatrix3d[9] = {1.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,1.0};

float  G3DIdentityMatrix4f[16] = {1.0f,0.0f,0.0f,0.0f,0.0f,1.0f,0.0f,0.0f,0.0f,0.0f,1.0f,0.0f,0.0f,0.0f,0.0f,1.0f};
double G3DIdentityMatrix4d[16] = {1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,1.0};

float  G3DDefaultPlane4f[4] = {0.0f,0.0f,1.0f,0.0f};
double G3DDefaultPlane4d[4] = {0.0,0.0,1.0,0.0};

/******************************************************************************
 *
 * Misc Functions Declaration
 *
 *****************************************************************************/

void G3DLinearInterpolateVector3fv(float res[3], CFLOAT a[3], CFLOAT b[3], CFLOAT lambda);
void G3DLinearInterpolateVector3dv(double res[3], CDOUBLE a[3], CDOUBLE b[3], CDOUBLE lambda);

float G3DDistanceFromPlanef(CFLOAT point[3],CFLOAT plane[4]);
double G3DDistanceFromPlaned(CDOUBLE point[3],CDOUBLE plane[4]);

int G3DCompareFloat(float a, float b, float tol);
int G3DCompareDouble(double a, double b, double tol);

void G3DFloatsFromDoubles(float* dest, CDOUBLE *src, const int size);
void G3DDoublesFromFloats(double* dest, CFLOAT *src, const int size);

/******************************************************************************
 *
 * Misc Functions Definition
 *
 *****************************************************************************/

__G3DI__ void G3DLinearInterpolateVector3fv(float res[3], CFLOAT a[3], CFLOAT b[3], CFLOAT lambda)
{
  float d1mlambda = 1.0f - lambda;

  if (d1mlambda == 0.0f) {
    res[0] = a[0];
    res[1] = a[1];
    res[2] = a[2];
  }
  else if (d1mlambda == 1.0f) {
    res[0] = b[0];
    res[1] = b[1];
    res[2] = b[2];
  }
  else {
    res[0] = lambda * a[0] + d1mlambda * b[0];
    res[1] = lambda * a[1] + d1mlambda * b[1];
    res[2] = lambda * a[2] + d1mlambda * b[2];
  }
}


__G3DI__ void G3DLinearInterpolateVector3dv(double res[3], CDOUBLE
a[3], CDOUBLE b[3], CDOUBLE lambda)
{
  double d1mlambda = 1.0 - lambda;

  if (d1mlambda == 0.0) {
    res[0] = a[0];
    res[1] = a[1];
    res[2] = a[2];
  }
  else if (d1mlambda == 1.0) {
    res[0] = b[0];
    res[1] = b[1];
    res[2] = b[2];
  }
  else {
    res[0] = lambda * a[0] + d1mlambda * b[0];
    res[1] = lambda * a[1] + d1mlambda * b[1];
    res[2] = lambda * a[2] + d1mlambda * b[2];
  }
}

__G3DI__ float G3DDistanceFromPlanef(CFLOAT point[3],CFLOAT plane[4])
{
  return plane[3] + (point[0]*plane[0]+point[1]*plane[1]+point[2]*plane[2]);
}

__G3DI__ double G3DDistanceFromPlaned(CDOUBLE point[3],CDOUBLE plane[4])
{
  return plane[3] + (point[0]*plane[0]+point[1]*plane[1]+point[2]*plane[2]);
}

__G3DI__ int G3DCompareFloat(float a, float b, float tol) 
{
  if ((a+tol) < b) return -1;
  if ((b+tol) < a) return 1;
  return 0;
}

__G3DI__ int G3DCompareDouble(double a, double b, double tol) 
{
  if ((a+tol) < b) return -1;
  if ((b+tol) < a) return 1;
  return 0;
}

__G3DI__ void G3DFloatsFromDoubles(float* dest, CDOUBLE *src, int size)
{
  while(size--) {
    *dest++ = (float)*src++;
  }
}

__G3DI__ void G3DDoublesFromFloats(double* dest, CFLOAT *src, int size)
{
  while(size--) {
    *dest++ = (double)*src++;
  }
}











