//-----------------------------------------------------------------------------
// Project            	GeometryKit
// Class		G3DVector2d
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DVector2d.m,v 1.5 2001/01/05 20:50:49 robert Exp $
//-----------------------------------------------------------------------------

#import "G3DVector2d.h"

#import "G3DFunctions.h"
#import "G3DDefs.h"
#import "G3DVectorFunc.h"
#import "G3DMatrixFunc.h"

@implementation G3DVector2d

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DVector2d *)vector2d
{
  return [[[G3DVector2d alloc] init] autorelease];
}

+ (G3DVector2d *)vector2dWithValues:(double *)vals
{
  return [[[G3DVector2d alloc] initWithElements:vals] autorelease];
}

+ (G3DVector2d *)vector2dWithX:(double)x y:(double)y
{
  return [[[G3DVector2d alloc] initWithX:x y:y] autorelease];
}

+ (G3DVector2d *)vector2dWithVector2d:(G3DVector2d *)vec
{
  return [[[G3DVector2d alloc] initWithVector:vec] autorelease];
}

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)initWithVector:(G3DVector2d *)vec
{
  if ((self = [super initWithX:[vec x] y:[vec y]])) {
  }
  return self;
}

//-----------------------------------------------------------------------------
// Math
//-----------------------------------------------------------------------------

- (G3DVector2d *)vectorByAdding:(G3DVector2d *)aVector
{
  G3DVector2d *newVector = [G3DVector2d vector2dWithVector2d:self];
  [newVector addTuple2d:aVector];
  return newVector;
}

- (G3DVector2d *)vectorBySubtracting:(G3DVector2d *)aVector
{
  G3DVector2d *newVector = [G3DVector2d vector2dWithVector2d:self];
  [newVector subTuple2d:aVector];
  return newVector;
}

- (G3DVector2d *)vectorByMultiplyingBy:(double)aScalar
{
  G3DVector2d *newVector = [G3DVector2d vector2dWithVector2d:self];
  [newVector multiplyBy:aScalar];
  return newVector;
}

- (G3DVector2d *)vectorByDividingBy:(double)aScalar
{
  G3DVector2d *newVector = [G3DVector2d vector2dWithVector2d:self];
  [newVector divideBy:aScalar];
  return newVector;
}

- (double)dotProduct:(G3DVector2d *)aVec
{
  double *b = [aVec elements];

  return (tuple[0] * b[0] + tuple[1] * b[1]); 
}

- (double)length
{
  return G3DLength2dv(tuple);
}

- (double)squaredLength
{
  return (tuple[0] * tuple[0] + tuple[1] * tuple[1]);
}

- (void)normalise
{
  G3DNormalise2dv(tuple,tuple);
}

- (G3DVector2d *)normalisedVector
{
  double t[2];

  G3DNormalise2dv(t,tuple);

  return [[[G3DVector2d alloc] initWithElements:t] autorelease];
}

@end






