//-----------------------------------------------------------------------------
// Project            	GeometryKit
// Class		G3DVector3f
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DVector3f.m,v 1.5 2001/01/05 20:50:49 robert Exp $
//-----------------------------------------------------------------------------

#import "G3DVector3f.h"

#import "G3DFunctions.h"
#import "G3DDefs.h"
#import "G3DVectorFunc.h"
#import "G3DMatrixFunc.h"

@implementation G3DVector3f

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DVector3f *)vector3f
{
  return [[[G3DVector3f alloc] init] autorelease];
}

+ (G3DVector3f *)vector3fWithValues:(float *)vals
{
  return [[[G3DVector3f alloc] initWithElements:vals] autorelease];
}

+ (G3DVector3f *)vector3fWithX:(float)x y:(float)y z:(float)z
{
  return [[[G3DVector3f alloc] initWithX:x y:y z:z] autorelease];
}

+ (G3DVector3f *)vector3fWithVector3f:(G3DVector3f *)vec
{
  return [[[G3DVector3f alloc] initWithVector:vec] autorelease];
}

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)initWithVector:(G3DVector3f *)vec
{
  if ((self = [super initWithX:[vec x] y:[vec y] z:[vec z]])) {
  }
  return self;
}

//-----------------------------------------------------------------------------
// Math
//-----------------------------------------------------------------------------

- (G3DVector3f *)vectorByAdding:(G3DVector3f *)aVector
{
  G3DVector3f* newVector = [G3DVector3f vector3fWithVector3f:self];
  [newVector addTuple3f:aVector];
  return newVector;
}

- (G3DVector3f *)vectorBySubtracting:(G3DVector3f *)aVector
{
  G3DVector3f* newVector = [G3DVector3f vector3fWithVector3f:self];
  [newVector subTuple3f:aVector];
  return newVector;
}

- (G3DVector3f *)vectorByMultiplyingBy:(float)aScalar
{
  G3DVector3f* newVector = [G3DVector3f vector3fWithVector3f:self];
  [newVector multiplyBy:aScalar];
  return newVector;
}

- (G3DVector3f *)vectorByDividingBy:(float)aScalar
{
  G3DVector3f* newVector = [G3DVector3f vector3fWithVector3f:self];
  [newVector divideBy:aScalar];
  return newVector;
}

- (float)dotProduct:(G3DVector3f *)aVec
{
  float *b = [aVec elements];

  return (tuple[0] * b[0] + tuple[1] * b[1] + tuple[2] * b[2]); 
}

- (void)crossProductWith:(G3DVector3f *)aVec and:(G3DVector3f *)bVec
{
  float *a = [aVec elements];
  float *b = [bVec elements];

  tuple[0] = a[1] * b[2] - a[2] * b[1];
  tuple[1] = a[2] * b[0] - a[0] * b[2];
  tuple[2] = a[0] * b[1] - a[1] * b[0];
}

- (float)length
{
  return G3DLength3fv(tuple);
}

- (float)squaredLength
{
  return (tuple[0] * tuple[0] + tuple[1] * tuple[1] + tuple[2] * tuple[2]);
}

- (void)normalise
{
  G3DNormalise3fv(tuple,tuple);
}

- (G3DVector3f *)normalisedVector
{
  float t[3];

  G3DNormalise3fv(t,tuple);

  return [[[G3DVector3f alloc] initWithElements:t] autorelease];
}

@end






