//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DArchivingInfo
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Apr 13 14:14:18 CEST 2000
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DArchivingInfo.m,v 1.5 2001/07/07 15:35:47 probert Exp $
//-----------------------------------------------------------------------------

#import <RenderKit/G3DArchivingInfo.h>
#import <RenderKit/G3DArchiver.h>
#import <RenderKit/G3DUnarchiver.h>

@implementation G3DArchivingInfo

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (id)archivingInfoWithClassName:(NSString *)name version:(int)_version
{
  G3DArchivingInfo *object = [[[self alloc] init] autorelease];

  object->className = [name copy];
  object->version = _version;

  return object;
}

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (NSString *)className	
{ 
  return className; 
}

- (int)version				
{ 
  return version; 
}

- (void)setIsWritten:(BOOL)flag	
{ 
  isWritten = flag; 
}

- (BOOL)isWritten			
{ 
  return isWritten; 
}

@end

@implementation NSString (SceneArchiving)

- (void)encodeWithSceneArchiver:(id)archiver
{
  [archiver encodeString:self named:@"string"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  return [unarchiver decodeStringNamed:@"string"];
}

- (Class)classForSceneArchiver
{
  return [NSString class];
}

@end

@implementation NSMutableString (SceneArchiving)

- (Class)classForSceneArchiver
{
  return [NSMutableString class];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver*)unarchiver
{
  return [[[unarchiver decodeStringNamed:@"string"] mutableCopy] autorelease];
}

@end

@implementation NSArray (SceneArchiving)

- (void)encodeWithSceneArchiver:(id)archiver
{
  [archiver encodeArray:self named:@"elements"];
}

- (Class)classForSceneArchiver
{
  return [NSMutableArray class];
}

@end

@implementation NSMutableArray (SceneArchiving)

- (id)initWithSceneUnarchiver:(G3DUnarchiver*)unarchiver
{
  id array = [unarchiver decodeArrayNamed:@"elements"];
  int i;
  int count;

  for (i = 0, count = [array count]; i < count; i++) {
    [self addObject:[array objectAtIndex:i]];
  }

  return self;
}

@end

@implementation NSDictionary (SceneArchiving)

- (void)encodeWithSceneArchiver:(id)archiver
{
  [archiver encodeDictionary:self named:@"elements"];
}

- (Class)classForSceneArchiver
{
  return [NSMutableDictionary class];
}

@end


@implementation NSMutableDictionary (SceneArchiving)

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  id dictionary = [unarchiver decodeDictionaryNamed:@"elements"];
  NSEnumerator *enumerator = [dictionary keyEnumerator];
  id key;
  id value;

  while ((key = [enumerator nextObject])) {
    value = [dictionary objectForKey:key];
    [self setObject:value forKey:key];
  }

  return self;
}

@end

@implementation NSData (SceneArchiving)

- (void)encodeWithSceneArchiver:(id)archiver
{
  [archiver encodeData:self named:@"data"];
}

- (Class)classForSceneArchiver
{
  return [NSMutableData class];
}

@end


@implementation NSMutableData (SceneArchiving)

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  id data = [unarchiver decodeDataNamed:@"data"];

  [self appendData:data];

  return self;
}

@end





