//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DAttributeNode
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Fri Mar 31 14:20:26 CEST 2000
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DAttributeNode.m,v 1.9 2001/01/05 20:50:51 robert Exp $
//-----------------------------------------------------------------------------

#import "G3DAttributeNode.h"
#import "G3DArchiver.h"
#import "G3DUnarchiver.h"

@implementation G3DAttributeNode

//-----------------------------------------------------------------------------
// Init and Free
//-----------------------------------------------------------------------------

- (id)init
{
  if ((self = [super init])) {
    [self setName:@"Generic Attribute"];
  }
  return self;
}

- (void)dealloc
{
  [name release];

  [super dealloc];
}

//-----------------------------------------------------------------------------
// Attribute information
//-----------------------------------------------------------------------------

- (void)setName:(NSString *)aName
{
  [name autorelease];
  name = [aName copy];
}

- (NSString *)name
{
  return name;
}

- (NSString *)description
{
  return [NSString stringWithFormat:@"%@ %x, thread %x, named %@",[self class], self, [NSThread currentThread],name];
}

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder
{
  [aCoder encodeObject:name];
}

- (id)initWithCoder:(NSCoder *)aCoder
{
  self = [super init];
  name = [[aCoder decodeObject] retain];
  return self;
}

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone
{
  id copy = [[[self class] allocWithZone:zone] init];

  [copy setName:name];
  
  return copy;
}

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeString:name named:@"name"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  name = [[unarchiver decodeStringNamed:@"name"] retain];
  return self;
}

@end






