//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DColour
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DColour.m,v 1.8 2001/01/05 20:50:51 robert Exp $
//-----------------------------------------------------------------------------

#import "G3DColour.h"
#import "G3DFunctions.h"
#import "G3DArchiver.h"
#import "G3DUnarchiver.h"

#import <GeometryKit/GeometryKit.h>

@implementation G3DColour

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DColour *)colour
{
  return [[[G3DColour alloc] initWithR:1.0 g:1.0 b:1.0 a:0.0] autorelease];
}

+ (G3DColour *)colourWithColour:(G3DColour *)col
{
  id newColour = [[[G3DColour alloc] init] autorelease];
  [newColour setValues:[col values]];
  return newColour;
}

+ (G3DColour *)colourWithR:(float)r g:(float)g b:(float)b a:(float)a
{
  return [[[G3DColour alloc] initWithR:r g:g b:b a:a] autorelease];
}

//-----------------------------------------------------------------------------
// Init
//-----------------------------------------------------------------------------

- (id)initWithR:(float)r g:(float)g b:(float)b a:(float)a
{
  if ((self = [super init])) {
    [self setName:@"Generic colour"];
    
    tuple[0] = CLAMP(r,0.0,1.0);
    tuple[1] = CLAMP(g,0.0,1.0);
    tuple[2] = CLAMP(b,0.0,1.0);
    tuple[3] = CLAMP(a,0.0,1.0);
  }
  return self;
}

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (float *)values
{
  return tuple;
}

- (void)setValues:(float *)values
{
  // Scale those values instead of clamping them!!!
  tuple[0] = CLAMP(values[0],0.0,1.0);
  tuple[1] = CLAMP(values[1],0.0,1.0);
  tuple[2] = CLAMP(values[2],0.0,1.0);
  tuple[3] = CLAMP(values[3],0.0,1.0);
}

- (void)setRed:(float)r
{
  tuple[0] = CLAMP(r,0.0,1.0);
}

- (float)red
{
  return tuple[0];
}

- (void)setGreen:(float)g
{
  tuple[1] = CLAMP(g,0.0,1.0);
}

- (float)green
{
  return tuple[1];
}

- (void)setBlue:(float)b
{
  tuple[2] = CLAMP(b,0.0,1.0);
}

- (float)blue
{
  return tuple[2];
}

- (void)setAlpha:(float)a
{
  tuple[3] = CLAMP(a,0.0,1.0);
}

- (float)alpha
{
  return tuple[3];
}

- (BOOL)isEqualToColour:(G3DColour *)oCol
{
  float *col = [oCol values];

  if(tuple[0]==col[0] && tuple[1]==col[1] && tuple[2]==col[2] && tuple[3]==col[3]) {
    return YES;
  }
  
  return NO;
}

- (NSString *)description
{
  return [NSString stringWithFormat:@"<%@ %x, thread %x>: r=%f g=%f b=%f a=%f",[self class],self,[NSThread currentThread],tuple[0],tuple[1],tuple[2],tuple[3]];
}

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder
{
  [super encodeWithCoder:aCoder];

  [aCoder encodeArrayOfObjCType:@encode(float) count:4 at:tuple];
}

- (id)initWithCoder:(NSCoder *)aCoder
{
  self = [super initWithCoder:aCoder];
  [aCoder decodeArrayOfObjCType:@encode(float) count:4 at:tuple];
  return self;
}

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone
{
  G3DColour *copy = [super copyWithZone:zone];

  [copy setValues:tuple];
  
  return copy;
}

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];

  [archiver encodeArrayOfTypeFloatAt:tuple count:4 named:@"tuple"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super initWithSceneUnarchiver:unarchiver];
  
  [unarchiver decodeArrayOfTypeFloatAt:tuple count:4 named:@"tuple"];

  return self;
}

@end






