//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DFog
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DFog.m,v 1.6 2001/01/05 20:50:51 robert Exp $
//-----------------------------------------------------------------------------

#import <RenderKit/G3DFog.h>

@implementation G3DFog

//-----------------------------------------------------------------------------
// Class method
//-----------------------------------------------------------------------------

+ (G3DFog *)fog
{
  return [[[G3DFog alloc] init] autorelease];
}

+ (G3DFog *)fogWithColour:(G3DColour *)col density:(double)dens start:(double)st end:(double)en fogMode:(G3D_FogMode)md
{
  id fog = [[G3DFog alloc] init];

  [fog setFogColour:col];
  [fog setDensity:dens];
  [fog setStart:st];
  [fog setEnd:en];
  [fog setFogMode:md];

  return [fog autorelease];
}

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)init
{
  if ((self = [super init])) {
    colour = [[G3DColour alloc] initWithR:1.0 g:1.0 b:1.0 a:0.0];
    mode = G3D_LINEAR_FOG;
    density = 0.5;
    start = 0.0;
    end = 100.0;
  }
  return self;
}

- (void)dealloc
{
  [colour release];
  [super dealloc];
}

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setFogColour:(G3DColour *)col
{
  [colour autorelease];
  colour = [col retain];
}

- (G3DColour *)fogColour
{
  return colour;
}

- (double)end
{
  return end;
}

- (void)setEnd:(double)val
{
  end = val;
}

- (double)start
{
  return start;
}

- (void)setStart:(double)val
{
  start = val;
}

- (double)density
{
  return density;
}

- (void)setDensity:(double)val
{
  density = val;
}

- (G3D_FogMode)fogMode
{
  return mode;
}

- (void)setFogMode:(G3D_FogMode)val
{
  mode = val;
}

- (void)cycleFogMode
{
  mode = (mode == G3D_LINEAR_FOG) ? G3D_EXP_FOG : G3D_LINEAR_FOG;
}

- (NSString *)description
{
  return [NSString stringWithFormat:@"<%@ %x, thread %x>: start=%f end=%f density=%f and colour %@",[self class],self,[NSThread currentThread],start,end,density,[colour description]];
}

@end






