//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DFunctions
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DFunctions.m,v 1.9 2001/01/05 20:50:51 robert Exp $
//-----------------------------------------------------------------------------

#import "G3DFunctions.h"

static Class NSStringClass = 0;

NSString *G3DStringFromFrustum(G3DFrustumValues fr);
NSString *G3DStringFromCoordinate(G3DCoordinate co);
NSString *G3DStringFromRange(G3DRange ra);
NSString *G3DStringFromBoundingSphere(G3DBoundingSphere bs);
NSString *G3DStringFromBoundingBox(G3DBoundingBox bb);

/*
NSString *NSStringFromCameraFlags(G3D_CameraFlags cf);
*/

NSString *G3DStringFromMatrix4f(float *mat);
NSString *G3DStringFromMatrix4d(double *mat);

NSString *G3DStringFromVectord(double *vec, int size);
NSString *G3DStringFromVectorf(float *vec, int size);

float G3DSwapZAxisMatrix[16] = {1.0, 0.0, 0.0, 0.0,
                               0.0, 0.0,-1.0, 0.0,
                               0.0, 1.0, 0.0, 0.0,
                               0.0, 0.0, 0.0, 1.0};

static inline void _initLookupCache()
{
  if (NSStringClass == 0) {
    NSStringClass = [NSString class];
  }
}

__G3DI__ NSString *G3DStringFromFrustum(G3DFrustumValues fr)
{
  _initLookupCache();

  return [NSStringClass stringWithFormat: @"{left=%f; right=%f; top=%f; bottom=%f; near=%f; far=%f; hfov=%f; vfov=%f}", fr.left, fr.right, fr.top, fr.bottom,fr.near,fr.far,fr.hfov,fr.vfov];
}

__G3DI__ NSString *G3DStringFromCoordinate(G3DCoordinate co)
{
  _initLookupCache();

  return [NSStringClass stringWithFormat: @"{posx=%f; posy=%f; posz=%f; dirx=%f; diry=%f; dirz=%f; upx=%f;upy=%f; upz=%f}", co.position[0],co.position[1],co.position[2],co.direction[0],co.direction[1],co.direction[2],co.up[0],co.up[1],co.up[2]];
}

__G3DI__ NSString *G3DStringFromRange(G3DRange ra)
{
  _initLookupCache();

  return [NSStringClass stringWithFormat: @"{offset=%d; length=%d}", ra.off, ra.length];
}
__G3DI__ NSString *G3DStringFromBoundingSphere(G3DBoundingSphere bs)
{
  _initLookupCache();

  return [NSStringClass stringWithFormat: @"{radius=%f; centerx=%f; centery=%f; centerz=%f}", bs.radius, bs.center[0],bs.center[1],bs.center[2]];
}

__G3DI__ NSString *G3DStringFromBoundingBox(G3DBoundingBox bb)
{
  _initLookupCache();

  return [NSStringClass stringWithFormat: @"{frontx=%f; fronty=%f; frontz=%f; backx=%f; backy=%f; backz=%f}", bb.front[0],bb.front[1],bb.front[2], bb.back[0],bb.back[1],bb.back[2]];
}

__G3DI__ NSString *G3DStringFromMatrix4f(float *mat)
{
  _initLookupCache();

  return [NSStringClass stringWithFormat: @"4x4 float matrix:\n\n%f %f %f %f \n%f %f %f %f \n%f %f %f %f \n%f %f %f %f \n",mat[0],mat[4],mat[8],mat[12],mat[1],mat[5],mat[9],mat[13],mat[2],mat[6],mat[10],mat[14],mat[3],mat[7],mat[11],mat[15]];
}

__G3DI__ NSString *G3DStringFromMatrix4d(double *mat)
{
  _initLookupCache();

  return [NSStringClass stringWithFormat: @"4x4 double matrix:\n\n%f %f %f %f \n%f %f %f %f \n%f %f %f %f \n%f %f %f %f \n",mat[0],mat[4],mat[8],mat[12],mat[1],mat[5],mat[9],mat[13],mat[2],mat[6],mat[10],mat[14],mat[3],mat[7],mat[11],mat[15]];
}

__G3DI__ NSString *G3DStringFromVectord(double *vec, int size)
{
  int i;
  NSString *str;

  _initLookupCache();

  str = [NSString stringWithFormat:@"Vector of size %d",size];

  for (i=0;i<size;i++) {
    str = [str stringByAppendingFormat:@", %f",vec[i]];
  }
  return [str stringByAppendingString:@"\n"];
}

__G3DI__ NSString *G3DStringFromVectorf(float *vec, int size)
{
  int i;
  NSString *str;

  _initLookupCache();

  str = [NSString stringWithFormat:@"Vector of size %d",size];

  for (i=0;i<size;i++) {
    str = [str stringByAppendingFormat:@", %f",vec[i]];
  }
  return str;
}

/*
__G3DI__ NSString *NSStringFromCameraFlags(G3D_CameraFlags cf)
{
  _initLookupCache();

  return [NSStringClass stringWithFormat: @"{degenerate=%d; basisChanged=%d; canRender=%d; usePreTM=%d; doFlush=%d; drawBackground=%d; isFullscreen=%d; doDepthTest=%d}", cf.degenerate, cf.basisChanged, cf.canRender, cf.usePreTM, cf.doFlush, cf.drawBackground, cf.isFullscreen, cf.doDepthTest];
}
*/



