//-----------------------------------------------------------------------------
// Project            	RenderKit
// Categories		G3DGeometryExtensions
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DGeometryExtensions.m,v 1.8 2001/01/05 20:50:51 robert Exp $
//-----------------------------------------------------------------------------

#import "G3DGeometryExtensions.h"

#import "G3DArchiver.h"
#import "G3DUnarchiver.h"

@implementation G3DBox (GeometryExtensions)

- (float)radius
{
  return 0.0;
}

- (G3DTuple3f *)center
{
  float c[3];
  float *_min = [min elements];
  float *_max = [max elements];

  c[0] = _max[0] - _min[0];
  c[1] = _max[1] - _min[1];
  c[2] = _max[2] - _min[2];

  return [G3DTuple3f tuple3fWithElements:c];
}

@end

@implementation G3DSphere (GeometryExtensions)

@end

@implementation G3DTuple2d (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeDoubleAt:tuple count:2 named:@"tuple2d"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeDoubleAt:tuple count:2 named:@"tuple2d"];
  return self;
}

@end

@implementation G3DTuple2f (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeFloatAt:tuple count:2 named:@"tuple2f"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeFloatAt:tuple count:2 named:@"tuple2f"];
  return self;
}

@end

@implementation G3DTuple3d (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeDoubleAt:tuple count:3 named:@"tuple3d"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeDoubleAt:tuple count:3 named:@"tuple3d"];
  return self;
}

@end

@implementation G3DTuple3f (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeFloatAt:tuple count:3 named:@"tuple3f"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeFloatAt:tuple count:3 named:@"tuple3f"];
  return self;
}

@end

@implementation G3DTuple4d (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeDoubleAt:tuple count:4 named:@"tuple4d"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeDoubleAt:tuple count:4 named:@"tuple4d"];
  return self;
}

@end

@implementation G3DTuple4f (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeFloatAt:tuple count:4 named:@"tuple4f"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeFloatAt:tuple count:4 named:@"tuple4f"];
  return self;
}

@end

@implementation G3DVector2d (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  return [super initWithSceneUnarchiver:unarchiver];
}

@end

@implementation G3DVector2f (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  return [super initWithSceneUnarchiver:unarchiver];
}

@end

@implementation G3DVector3d (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  return [super initWithSceneUnarchiver:unarchiver];
}

@end

@implementation G3DVector3f (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  return [super initWithSceneUnarchiver:unarchiver];
}

@end

@implementation G3DVector4d (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  return [super initWithSceneUnarchiver:unarchiver];
}

@end

@implementation G3DVector4f (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  return [super initWithSceneUnarchiver:unarchiver];
}

@end

@implementation G3DQuaternion (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  return [super initWithSceneUnarchiver:unarchiver];
}

@end

@implementation G3DVector (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeInt:_size named:@"size"];
  [archiver encodeArrayOfTypeDoubleAt:vector count:_size named:@"vector"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeInt:&_size named:@"size"];
  vector = malloc(_size * sizeof(double));
  [unarchiver decodeArrayOfTypeDoubleAt:vector count:_size named:@"vector"];
  return self;
}

@end

@implementation G3DMatrix3d (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeDoubleAt:matrix count:9 named:@"matrix"];
  [archiver encodeInt:_mode named:@"mode"];
  [archiver encodeArrayOfTypeDoubleAt:_t count:9 named:@"transposed"];
  [archiver encodeArrayOfTypeDoubleAt:_i count:9 named:@"inverted"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeDoubleAt:matrix count:9 named:@"matrix"];
  [unarchiver decodeInt:&_mode named:@"mode"];
  [unarchiver decodeArrayOfTypeDoubleAt:_t count:9 named:@"transposed"];
  [unarchiver decodeArrayOfTypeDoubleAt:_i count:9 named:@"inverted"];
  return self;
}

@end

@implementation G3DMatrix3f (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeFloatAt:matrix count:9 named:@"matrix"];
  [archiver encodeInt:_mode named:@"mode"];
  [archiver encodeArrayOfTypeFloatAt:_t count:9 named:@"transposed"];
  [archiver encodeArrayOfTypeFloatAt:_i count:9 named:@"inverted"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeFloatAt:matrix count:9 named:@"matrix"];
  [unarchiver decodeInt:&_mode named:@"mode"];
  [unarchiver decodeArrayOfTypeFloatAt:_t count:9 named:@"transposed"];
  [unarchiver decodeArrayOfTypeFloatAt:_i count:9 named:@"inverted"];
  return self;
}

@end

@implementation G3DMatrix4d (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeDoubleAt:matrix count:16 named:@"matrix"];
  [archiver encodeInt:_mode named:@"mode"];
  [archiver encodeArrayOfTypeDoubleAt:_t count:16 named:@"transposed"];
  [archiver encodeArrayOfTypeDoubleAt:_i count:16 named:@"inverted"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeDoubleAt:matrix count:16 named:@"matrix"];
  [unarchiver decodeInt:&_mode named:@"mode"];
  [unarchiver decodeArrayOfTypeDoubleAt:_t count:16 named:@"transposed"];
  [unarchiver decodeArrayOfTypeDoubleAt:_i count:16 named:@"inverted"];
  return self;
}

@end

@implementation G3DMatrix4f (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeFloatAt:matrix count:16 named:@"matrix"];
  [archiver encodeInt:_mode named:@"mode"];
  [archiver encodeArrayOfTypeFloatAt:_t count:16 named:@"transposed"];
  [archiver encodeArrayOfTypeFloatAt:_i count:16 named:@"inverted"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeFloatAt:matrix count:16 named:@"matrix"];
  [unarchiver decodeInt:&_mode named:@"mode"];
  [unarchiver decodeArrayOfTypeFloatAt:_t count:16 named:@"transposed"];
  [unarchiver decodeArrayOfTypeFloatAt:_i count:16 named:@"inverted"];
  return self;
}

@end

@implementation G3DMatrix (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeInt:_rows named:@"rows"];
  [archiver encodeInt:_columns named:@"columns"];
  [archiver encodeArrayOfTypeDoubleAt:matrix count:_rows*_columns named:@"matrix"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeInt:&_rows named:@"rows"];
  [unarchiver decodeInt:&_columns named:@"columns"];
  matrix = malloc(_rows * _columns * sizeof(double));
  [unarchiver decodeArrayOfTypeDoubleAt:matrix count:_rows*_columns named:@"matrix"];
  return self;
}

@end

@implementation G3DBezier (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArray:_controlPoints named:@"controlPoints"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  _controlPoints = [NSMutableArray arrayWithArray:[unarchiver decodeArrayNamed:@"controlPoints"]];
  return self;
}

@end

@implementation G3DBox (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeObject:min named:@"min"];
  [archiver encodeObject:max named:@"max"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  min = [unarchiver decodeObjectNamed:@"min"];
  max = [unarchiver decodeObjectNamed:@"min"];
  return self;
}

@end

@implementation G3DLine (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeObject:origin named:@"origin"];
  [archiver encodeObject:direction named:@"direction"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  origin = [unarchiver decodeObjectNamed:@"origin"];
  direction = [unarchiver decodeObjectNamed:@"direction"];
  return self;
}

@end

@implementation G3DPlane (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeFloatAt:values count:4 named:@"values"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeFloatAt:values count:4 named:@"values"];
  return self;
}

@end

@implementation G3DSphere (GeometryArchiving)

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [archiver encodeArrayOfTypeFloatAt:center count:3 named:@"center"];
  [archiver encodeFloat:radius named:@"radius"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super init];
  [unarchiver decodeArrayOfTypeFloatAt:center count:3 named:@"center"];
  [unarchiver decodeFloat:&radius named:@"radius"];
  return self;
}

@end
