//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DLeaf
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Wed Nov 24 11:10:03 CET 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DLeaf.m,v 1.8 2001/01/05 20:50:52 robert Exp $
//-----------------------------------------------------------------------------

#import "G3DLeaf.h"
#import "G3DArchiver.h"
#import "G3DUnarchiver.h"
#import "G3DTransform.h"
#import "G3DTypes.h"
#import "G3DDefines.h"

#import <GeometryKit/GeometryKit.h>

@implementation G3DLeaf

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)init
{
  if ((self = [super init])) {
    _useDisplayList = NO;
  }
  return self;
}

//-----------------------------------------------------------------------------
// Positioning
//-----------------------------------------------------------------------------

- (void)setPositionWithValues:(float *)pos
{
  [transformation setTranslationValues:pos];
  _transfIsDirty = YES;
}

- (void)setPositionWithTuple:(G3DTuple3f *)pos
{
  [transformation setTranslationValues:[pos elements]];
}

- (void)getPosition:(G3DTuple3f **)pos
{
  float pos_arr[3];

  [transformation getTranslationValues:pos_arr];
  [*pos setElements:pos_arr];
}

//-----------------------------------------------------------------------------
// Optimisiation
//-----------------------------------------------------------------------------

- (void)setUseDisplayList:(BOOL)flag
{
  _useDisplayList = flag;
}

- (BOOL)useDisplayList
{
  return _useDisplayList;
}

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder
{
  [super encodeWithCoder:aCoder];

  [aCoder encodeValueOfObjCType:@encode(BOOL) at:&_useDisplayList];
}

- (id)initWithCoder:(NSCoder *)aCoder
{
  self = [super initWithCoder:aCoder];

  [aCoder decodeValueOfObjCType:@encode(BOOL) at:&_useDisplayList];

  return self;
}

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone
{
  id copy = [super copyWithZone:zone];

  [copy setUseDisplayList:_useDisplayList];
  
  return copy;
}

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];

  [archiver encodeBOOL:_useDisplayList named:@"_useDisplayList"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super initWithSceneUnarchiver:unarchiver];
  
  [unarchiver decodeBOOL:&_useDisplayList named:@"_useDisplayList"];

  return self;
}

@end






