//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DState
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DState.m,v 1.19 2001/07/15 12:39:24 probert Exp $
//-----------------------------------------------------------------------------

#import "G3DState.h"

#import <GeometryKit/G3DDefs.h>

#import "G3DTexture.h"
#import "G3DCamera.h"
#import "G3DDefines.h"
#import "G3DMaterial.h"
#import "G3DColour.h"
#import "G3DArchiver.h"
#import "G3DUnarchiver.h"

NSString *G3DStateException = @"G3DStateException";

/*
 * State handling partially based on S.Baker's PLIB
 *
 */

#define E0  glEnable ( GL_TEXTURE_2D      ) ;
#define E1  glEnable ( GL_CULL_FACE       ) ;
#define E2  glEnable ( GL_COLOR_MATERIAL  ) ;
#define E3  glEnable ( GL_BLEND           ) ;
#define E4  glEnable ( GL_ALPHA_TEST      ) ;
#define E5  glEnable ( GL_LIGHTING        ) ;

#define D0  glDisable ( GL_TEXTURE_2D     ) ;
#define D1  glDisable ( GL_CULL_FACE      ) ;
#define D2  glDisable ( GL_COLOR_MATERIAL ) ;
#define D3  glDisable ( GL_BLEND          ) ;
#define D4  glDisable ( GL_ALPHA_TEST     ) ;
#define D5  glDisable ( GL_LIGHTING       ) ;

/*
 * Any possible and useful combinations
 *
 */

static void enable_00 () {                }
static void enable_01 () { E0             }
static void enable_02 () {    E1          }
static void enable_03 () { E0 E1          }
static void enable_04 () {       E2       }
static void enable_05 () { E0    E2       }
static void enable_06 () {    E1 E2       }
static void enable_07 () { E0 E1 E2       }
static void enable_08 () {          E3    }
static void enable_09 () { E0       E3    }
static void enable_10 () {    E1    E3    }
static void enable_11 () { E0 E1    E3    }
static void enable_12 () {       E2 E3    }
static void enable_13 () { E0    E2 E3    }
static void enable_14 () {    E1 E2 E3    }
static void enable_15 () { E0 E1 E2 E3    }
static void enable_16 () {             E4 }
static void enable_17 () { E0          E4 }
static void enable_18 () {    E1       E4 }
static void enable_19 () { E0 E1       E4 }
static void enable_20 () {       E2    E4 }
static void enable_21 () { E0    E2    E4 }
static void enable_22 () {    E1 E2    E4 }
static void enable_23 () { E0 E1 E2    E4 }
static void enable_24 () {          E3 E4 }
static void enable_25 () { E0       E3 E4 }
static void enable_26 () {    E1    E3 E4 }
static void enable_27 () { E0 E1    E3 E4 }
static void enable_28 () {       E2 E3 E4 }
static void enable_29 () { E0    E2 E3 E4 }
static void enable_30 () {    E1 E2 E3 E4 }
static void enable_31 () { E0 E1 E2 E3 E4 }

static void enable_32 () {                E5 }
static void enable_33 () { E0             E5 }
static void enable_34 () {    E1          E5 }
static void enable_35 () { E0 E1          E5 }
static void enable_36 () {       E2       E5 }
static void enable_37 () { E0    E2       E5 }
static void enable_38 () {    E1 E2       E5 }
static void enable_39 () { E0 E1 E2       E5 }
static void enable_40 () {          E3    E5 }
static void enable_41 () { E0       E3    E5 }
static void enable_42 () {    E1    E3    E5 }
static void enable_43 () { E0 E1    E3    E5 }
static void enable_44 () {       E2 E3    E5 }
static void enable_45 () { E0    E2 E3    E5 }
static void enable_46 () {    E1 E2 E3    E5 }
static void enable_47 () { E0 E1 E2 E3    E5 }
static void enable_48 () {             E4 E5 }
static void enable_49 () { E0          E4 E5 }
static void enable_50 () {    E1       E4 E5 }
static void enable_51 () { E0 E1       E4 E5 }
static void enable_52 () {       E2    E4 E5 }
static void enable_53 () { E0    E2    E4 E5 }
static void enable_54 () {    E1 E2    E4 E5 }
static void enable_55 () { E0 E1 E2    E4 E5 }
static void enable_56 () {          E3 E4 E5 }
static void enable_57 () { E0       E3 E4 E5 }
static void enable_58 () {    E1    E3 E4 E5 }
static void enable_59 () { E0 E1    E3 E4 E5 }
static void enable_60 () {       E2 E3 E4 E5 }
static void enable_61 () { E0    E2 E3 E4 E5 }
static void enable_62 () {    E1 E2 E3 E4 E5 }
static void enable_63 () { E0 E1 E2 E3 E4 E5 }

static void disable_00 () {                }
static void disable_01 () { D0             }
static void disable_02 () {    D1          }
static void disable_03 () { D0 D1          }
static void disable_04 () {       D2       }
static void disable_05 () { D0    D2       }
static void disable_06 () {    D1 D2       }
static void disable_07 () { D0 D1 D2       }
static void disable_08 () {          D3    }
static void disable_09 () { D0       D3    }
static void disable_10 () {    D1    D3    }
static void disable_11 () { D0 D1    D3    }
static void disable_12 () {       D2 D3    }
static void disable_13 () { D0    D2 D3    }
static void disable_14 () {    D1 D2 D3    }
static void disable_15 () { D0 D1 D2 D3    }
static void disable_16 () {             D4 }
static void disable_17 () { D0          D4 }
static void disable_18 () {    D1       D4 }
static void disable_19 () { D0 D1       D4 }
static void disable_20 () {       D2    D4 }
static void disable_21 () { D0    D2    D4 }
static void disable_22 () {    D1 D2    D4 }
static void disable_23 () { D0 D1 D2    D4 }
static void disable_24 () {          D3 D4 }
static void disable_25 () { D0       D3 D4 }
static void disable_26 () {    D1    D3 D4 }
static void disable_27 () { D0 D1    D3 D4 }
static void disable_28 () {       D2 D3 D4 }
static void disable_29 () { D0    D2 D3 D4 }
static void disable_30 () {    D1 D2 D3 D4 }
static void disable_31 () { D0 D1 D2 D3 D4 }

static void disable_32 () {                D5 }
static void disable_33 () { D0             D5 }
static void disable_34 () {    D1          D5 }
static void disable_35 () { D0 D1          D5 }
static void disable_36 () {       D2       D5 }
static void disable_37 () { D0    D2       D5 }
static void disable_38 () {    D1 D2       D5 }
static void disable_39 () { D0 D1 D2       D5 }
static void disable_40 () {          D3    D5 }
static void disable_41 () { D0       D3    D5 }
static void disable_42 () {    D1    D3    D5 }
static void disable_43 () { D0 D1    D3    D5 }
static void disable_44 () {       D2 D3    D5 }
static void disable_45 () { D0    D2 D3    D5 }
static void disable_46 () {    D1 D2 D3    D5 }
static void disable_47 () { D0 D1 D2 D3    D5 }
static void disable_48 () {             D4 D5 }
static void disable_49 () { D0          D4 D5 }
static void disable_50 () {    D1       D4 D5 }
static void disable_51 () { D0 D1       D4 D5 }
static void disable_52 () {       D2    D4 D5 }
static void disable_53 () { D0    D2    D4 D5 }
static void disable_54 () {    D1 D2    D4 D5 }
static void disable_55 () { D0 D1 D2    D4 D5 }
static void disable_56 () {          D3 D4 D5 }
static void disable_57 () { D0       D3 D4 D5 }
static void disable_58 () {    D1    D3 D4 D5 }
static void disable_59 () { D0 D1    D3 D4 D5 }
static void disable_60 () {       D2 D3 D4 D5 }
static void disable_61 () { D0    D2 D3 D4 D5 }
static void disable_62 () {    D1 D2 D3 D4 D5 }
static void disable_63 () { D0 D1 D2 D3 D4 D5 }

void (*__G3DEnableTable[64])() =
{
  enable_00, enable_01, enable_02, enable_03, enable_04,
  enable_05, enable_06, enable_07, enable_08, enable_09,
  enable_10, enable_11, enable_12, enable_13, enable_14,
  enable_15, enable_16, enable_17, enable_18, enable_19,
  enable_20, enable_21, enable_22, enable_23, enable_24,
  enable_25, enable_26, enable_27, enable_28, enable_29,
  enable_30, enable_31,

  enable_32, enable_33, enable_34, enable_35, enable_36,
  enable_37, enable_38, enable_39, enable_40, enable_41,
  enable_42, enable_43, enable_44, enable_45, enable_46,
  enable_47, enable_48, enable_49, enable_50, enable_51,
  enable_52, enable_53, enable_54, enable_55, enable_56,
  enable_57, enable_58, enable_59, enable_60, enable_61,
  enable_62, enable_63
} ;

void (*__G3DDisableTable[64])() =
{
  disable_00, disable_01, disable_02, disable_03, disable_04,
  disable_05, disable_06, disable_07, disable_08, disable_09,
  disable_10, disable_11, disable_12, disable_13, disable_14,
  disable_15, disable_16, disable_17, disable_18, disable_19,
  disable_20, disable_21, disable_22, disable_23, disable_24,
  disable_25, disable_26, disable_27, disable_28, disable_29,
  disable_30, disable_31,

  disable_32, disable_33, disable_34, disable_35, disable_36,
  disable_37, disable_38, disable_39, disable_40, disable_41,
  disable_42, disable_43, disable_44, disable_45, disable_46,
  disable_47, disable_48, disable_49, disable_50, disable_51,
  disable_52, disable_53, disable_54, disable_55, disable_56,
  disable_57, disable_58, disable_59, disable_60, disable_61,
  disable_62, disable_63
} ;

@interface G3DState (Private)

- (void)setEnable:(int)en;
- (int)enable;

@end

@implementation G3DState (Private)

- (void)setEnable:(int)en
{
  enable = en;
}

- (int)enable
{
  return enable;
}

@end

@implementation G3DState

void enableState(G3DState *self, int s_feature)
{
  if ((self->enable & (1<<s_feature)) == 0) {
    self->enable |=  (1<<s_feature);
    self->ignore &= ~(1<<s_feature); 
  }
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (void)initialize
{
  static BOOL isInitialised = NO;

  if (!isInitialised) {
    isInitialised = YES;
  }
}

+ (G3DState *)state
{
  return [[[G3DState alloc] init] autorelease];
}

+ (G3DState *)currentState
{
  return [[G3DCamera activeCamera] state];
}

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init
{
  return [self initWithMaterial:nil];
}

- (id)initWithMaterial:(G3DMaterial *)m
{
  if ((self = [super init])) {
    [self setName:@"Generic state"];

    // Set some defaults...
    ignore = 0xFFFFFFFF;

    shadeMode = G3D_Smooth;
    ignore &= ~(1<<G3D_SHADE_MODEL); 

    ignore &= ~(1<<G3D_DIFFUSE); 
    ignore &= ~(1<<G3D_AMBIENT); 
    ignore &= ~(1<<G3D_EMISSION); 
    //ignore &= ~(1<<G3D_SPECULAR); 
    ignore &= ~(1<<G3D_SHININESS); 

    if (!m) {
      material = [[G3DMaterial alloc] initWithState:self];
    }
    else {
      material = [m copy];
      [material setState:self];
    }

    alphaFunc = G3D_Greater;

    [self enableFeature:G3D_Culling];
    [self enableFeature:G3D_ColourMaterial];
    [self enableFeature:G3D_AlphaTest];

    [self disableFeature:G3D_Lighting];
    [self disableFeature:G3D_Texturing];
    [self disableFeature:G3D_Blending];
    
    textureHandle = 0;
  }
  return self;
}

- (void)dealloc
{
  [material release];

  [super dealloc];
}

//-----------------------------------------------------------------------------
// State methods
//-----------------------------------------------------------------------------

- (void)activate
{
  G3DState *_c_st = [[G3DCamera activeCamera] state];
  int _tbl_ind_on;
  int _tbl_ind_off;
  int _c_enable;

  DLOG(@"*** <%@,%x>: activate ***\n",[self class],self);

  /*
   * Apply changes to the material values only if needed!
   *
   */
  
  if (~ignore & ((1 << G3D_COLOR_MATERIAL) |
		 (1 << G3D_DIFFUSE)        |
		 (1 << G3D_AMBIENT)        |
		 (1 << G3D_SPECULAR)       |
		 (1 << G3D_EMISSION)       |
		 (1 << G3D_SHININESS)) ) {
    G3DMaterial *_c_st_m = [_c_st material];
    int _face = [_c_st_m face];

    /*
     * This could be cached!
     *
     */

    G3D_MaterialMode _mode = [material materialMode];
    double _sh = [material shininess];
    G3DColour *_s_col = [material specularColour];
    G3DColour *_e_col = [material emissiveColour];
    G3DColour *_a_col = [material ambientColour];
    G3DColour *_d_col = [material diffuseColour];

    DLOG(@"Applying the state node %@: %x, %x",[self class], self, [NSThread currentThread]);

    if (~ignore & (1 << G3D_COLOR_MATERIAL)) {
      glColorMaterial(_face,(GLenum)_mode);
      [_c_st_m setMaterialMode:_mode];
      DLOG(@"%x forces G3D_COLOR_MATERIAL",self);
    }

    if (~ignore & (1 << G3D_SHININESS)) {
      glMaterialf(_face,GL_SHININESS,_sh);
      [_c_st_m setShininess:_sh];
      DLOG(@"%x forces G3D_SHININESS",self);
    }

    if (~ignore & (1 << G3D_SPECULAR)) {
      glMaterialfv(_face,GL_SPECULAR,[_s_col values]);
      [_c_st_m setSpecularColour:_s_col];
      DLOG(@"%x forces G3D_SPECULAR",self);
    }

    if (~ignore & (1 << G3D_EMISSION)) {
      glMaterialfv(_face,GL_EMISSION,[_e_col values]);
      [_c_st_m setEmissiveColour:_e_col];
      DLOG(@"%x forces G3D_EMISSION",self);
    }

    if (~ignore & (1 << G3D_AMBIENT)) {
      glMaterialfv(_face,GL_AMBIENT,[_a_col values]);
      [_c_st_m setAmbientColour:_a_col];
      DLOG(@"%x forces G3D_AMBIENT",self);
    }

    if (~ignore & (1 << G3D_DIFFUSE)) {
      glMaterialfv(_face,GL_DIFFUSE,[_d_col values]);
      [_c_st_m setDiffuseColour:_d_col];
      DLOG(@"%x forces G3D_DIFFUSE",self);
    }
  }

  /*
   * Do it using the predefined tables
   *
   */

  _c_enable = [_c_st enable];

  _tbl_ind_on  = ~ignore &  enable & 0x3F;
  _tbl_ind_off = ~ignore & ~enable & 0x3F;

  DLOG(@"State %x: _tbl_ind_on = %d, _tbl_ind_off = %d",self,_tbl_ind_on,_tbl_ind_off);

  (*(__G3DEnableTable [_tbl_ind_on]))();
  (*(__G3DDisableTable[_tbl_ind_off]))();
  
  _c_enable |= _tbl_ind_on;
  _c_enable &= ~_tbl_ind_off;

  [_c_st setEnable:_c_enable];

  /*
   * Shading and texture handling
   *
   */

  if (~ignore & (1 << G3D_TEXTURE)) {
    glBindTexture(GL_TEXTURE_2D, textureHandle);
    [_c_st setTextureHandle:textureHandle];
    DLOG(@"%x forces G3D_TEXTURE with handle %d",self,textureHandle);
  }
  
  if (~ignore & (1 << G3D_SHADE_MODEL)) {
    glShadeModel(shadeMode);
    [_c_st setShadeMode:shadeMode];
    DLOG(@"%x forces G3D_SHADE_MODEL",self);
  }
  
  if (~ignore & (1 << G3D_ALPHA_TEST)) {
    glAlphaFunc(alphaFunc,alphaClamp);
    [_c_st setAlphaClamp:alphaClamp]; // I don't cache the func....
    DLOG(@"%x forces G3D_ALPHA_TEST",self);
  }
}

- (void)activateIfNeeded
{
  G3DState *_c_st = [[G3DCamera activeCamera] state];
  int _tbl_ind_on;
  int _tbl_ind_off;
  int _c_enable;

  DLOG(@"*** <%@,%x>: activateIfNeeded ***\n",[self class],self);

  /*
   * Apply changes to the material values only if needed!
   *
   */
  
  if (~ignore & ((1 << G3D_COLOR_MATERIAL) |
		(1 << G3D_DIFFUSE)        |
		(1 << G3D_AMBIENT)        |
		(1 << G3D_SPECULAR)       |
		(1 << G3D_EMISSION)       |
		(1 << G3D_SHININESS)) ) {
    BOOL didSwitch = NO;
    G3DMaterial *_c_st_m = [_c_st material];
    int _face = [_c_st_m face];
    
    /*
     * This could be cached!
     *
     */

    G3D_MaterialMode _mode = [material materialMode];
    double _sh = [material shininess];
    G3DColour *_s_col = [material specularColour];
    G3DColour *_e_col = [material emissiveColour];
    G3DColour *_a_col = [material ambientColour];
    G3DColour *_d_col = [material diffuseColour];

    DLOG(@"Applying state on %x, class %@,thread %x",self, [self class], [NSThread currentThread]);

    if (~ignore & (1 << G3D_COLOR_MATERIAL) && [_c_st_m materialMode] != _mode){
      glColorMaterial(_face,(GLenum)_mode);
      [_c_st_m setMaterialMode:_mode];
      didSwitch = YES;
      DLOG(@"%x handles G3D_COLOR_MATERIAL",self);
    }

    if (~ignore & (1 << G3D_SHININESS) && [_c_st_m shininess] != _sh) {
      glMaterialf(_face,GL_SHININESS,_sh);
      [_c_st_m setShininess:_sh];
      DLOG(@"%x handles G3D_SHININESS",self);
    }

    if (~ignore & (1 << G3D_SPECULAR) && ( didSwitch || ![[_c_st_m specularColour] isEqualToColour:_s_col])) {
      glMaterialfv(_face,GL_SPECULAR,[_s_col values]);
      [_c_st_m setSpecularColour:_s_col];
      DLOG(@"%x handles G3D_SPECULAR",self);
    }

    if (~ignore & (1 << G3D_EMISSION) && ( didSwitch || ![[_c_st_m emissiveColour] isEqualToColour:_e_col])) {
      glMaterialfv(_face,GL_EMISSION,[_e_col values]);
      [_c_st_m setEmissiveColour:_e_col];
      DLOG(@"%x handles G3D_EMISSION",self);
    }
    
    if (~ignore & (1 << G3D_AMBIENT) && ( didSwitch || ![[_c_st_m ambientColour] isEqualToColour:_a_col])) {
      glMaterialfv(_face,GL_AMBIENT,[_a_col values]);
      [_c_st_m setAmbientColour:_a_col];
      DLOG(@"%x handles G3D_AMBIENT",self);
    }

    if (~ignore & (1 << G3D_DIFFUSE) && ( didSwitch || ![[_c_st_m diffuseColour] isEqualToColour:_d_col])) {
      glMaterialfv(_face,GL_DIFFUSE,[_d_col values]);
      [_c_st_m setDiffuseColour:_d_col];
      DLOG(@"%x handles G3D_DIFFUSE",self);
    }
  }

  /*
   * Do it using the predefined tables
   *
   */

  _c_enable = [_c_st enable];

  _tbl_ind_on  = ~ignore &  enable & ~_c_enable & 0x3F;
  _tbl_ind_off = ~ignore & ~enable &  _c_enable & 0x3F;

  DLOG(@"%x sets _tbl_ind_on to %d and _tbl_ind_off to %d",self,_tbl_ind_on,_tbl_ind_off);

  (*(__G3DEnableTable [_tbl_ind_on]))();
  (*(__G3DDisableTable[_tbl_ind_off]))();

  _c_enable |=  _tbl_ind_on;
  _c_enable &= ~_tbl_ind_off;

  [_c_st setEnable:_c_enable];
  
  /*
   * Shading and texture handling
   *
   */

  if (~ignore & (1 << G3D_TEXTURE) && [_c_st textureHandle]!=textureHandle) {
    glBindTexture(GL_TEXTURE_2D, textureHandle);
    [_c_st setTextureHandle:textureHandle];
    DLOG(@"%x handles G3D_TEXTURE with handle %d",self,textureHandle);
  }
  
  if (~ignore & (1 << G3D_SHADE_MODEL) && [_c_st shadeMode] != shadeMode) {
    glShadeModel(shadeMode);
    [_c_st setShadeMode:shadeMode];
    DLOG(@"%x handles G3D_SHADE_MODEL",self);
  }
 
  if (~ignore & (1 << G3D_ALPHA_TEST) && [_c_st alphaClamp] != alphaClamp) {
    glAlphaFunc(alphaFunc,alphaClamp);
    [_c_st setAlphaClamp:alphaClamp]; // I don't cache the func....
    DLOG(@"%x handles G3D_ALPHA_TEST",self);
  }
}

- (BOOL)isTranslucent
{
  return isTranslucent;
}

- (void)setIsTranslucent:(BOOL)yn
{
  isTranslucent = yn;
}

- (void)setShadeMode:(G3D_ShadeMode)mode
{
  shadeMode = mode;
}

- (G3D_ShadeMode)shadeMode
{
  return shadeMode;
}

- (void)setAlphaFunc:(G3D_AlphaFunc)func
{
  alphaFunc = func;
}

- (G3D_AlphaFunc)alphaFunc
{
  return alphaFunc;
}

- (void)setAlphaClamp:(float)val
{
  alphaClamp = CLAMP(val,0.0,1.0);
}

- (float)alphaClamp
{
  return alphaClamp;
}

//-----------------------------------------------------------------------------
// Activate and deactivate
//-----------------------------------------------------------------------------

- (BOOL)isFeatureEnabled:(G3D_RenderFeature)md;
{
  switch (md) {
  case G3D_Texturing:
    return enable & (1<<G3D_TEXTURE_EN);
    break;
  case G3D_Culling:
    return enable & (1<<G3D_CULL_FACE_EN);
    break;
  case G3D_ColourMaterial:
    return enable & (1<<G3D_COLOR_MATERIAL_EN);
    break;
  case G3D_Blending:
    return enable & (1<<G3D_BLEND_EN);
    break;
  case G3D_AlphaTest:
    return enable & (1<<G3D_ALPHA_TEST_EN);
    break;
  case G3D_Lighting:
    return enable & (1<<G3D_LIGHTING_EN);
    break;
  default:
    [NSException raise:G3DStateException format:@"Unsupported feature!"];
    break;
  }
  return NO;
}

- (void)enableFeature:(G3D_RenderFeature)md;
{
  switch (md) {
  case G3D_Texturing:
    enable |= (1<<G3D_TEXTURE_EN);
    ignore &= ~(1<<G3D_TEXTURE_EN); 
    break;
  case G3D_Culling:
    enable |= (1<<G3D_CULL_FACE_EN);
    ignore &= ~(1<<G3D_CULL_FACE_EN); 
    break;
  case G3D_ColourMaterial:
    enable |= (1<<G3D_COLOR_MATERIAL_EN);
    ignore &= ~(1<<G3D_COLOR_MATERIAL_EN); 
    break;
  case G3D_Blending:
    enable |= (1<<G3D_BLEND_EN);
    ignore &= ~(1<<G3D_BLEND_EN); 
    break;
  case G3D_AlphaTest:
    enable |= (1<<G3D_ALPHA_TEST_EN);
    ignore &= ~(1<<G3D_ALPHA_TEST_EN); 
    break;
  case G3D_Lighting:
    enable |= (1<<G3D_LIGHTING_EN);
    ignore &= ~(1<<G3D_LIGHTING_EN); 
    break;
  default:
    [NSException raise:G3DStateException format:@"Unsupported feature!"];
    break;
  }
}

- (void)disableFeature:(G3D_RenderFeature)md;
{
  switch (md) {
  case G3D_Texturing:
    enable &= ~(1<<G3D_TEXTURE_EN);
    ignore &= ~(1<<G3D_TEXTURE_EN); 
    break;
  case G3D_Culling:
    enable &= ~(1<<G3D_CULL_FACE_EN);
    ignore &= ~(1<<G3D_CULL_FACE_EN); 
    break;
  case G3D_ColourMaterial:
    enable &= ~(1<<G3D_COLOR_MATERIAL_EN);
    ignore &= ~(1<<G3D_COLOR_MATERIAL_EN); 
    break;
  case G3D_Blending:
    enable &= ~(1<<G3D_BLEND_EN);
    ignore &= ~(1<<G3D_BLEND_EN); 
    break;
  case G3D_AlphaTest:
    enable &= ~(1<<G3D_ALPHA_TEST_EN);
    ignore &= ~(1<<G3D_ALPHA_TEST_EN); 
    break;
  case G3D_Lighting:
    enable &= ~(1<<G3D_LIGHTING_EN);
    ignore &= ~(1<<G3D_LIGHTING_EN); 
    break;
  default:
    break;
  }
}


//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setMaterial:(G3DMaterial *)mat
{
  [material autorelease];
  material = [mat copy];

  [material setState:self];
}

- (G3DMaterial *)material
{
  return material;
}

- (void)setTextureHandle:(GLuint)handle
{
  textureHandle = handle;

  // Enable the texture flag!
  ignore &= ~(1<<G3D_TEXTURE); 
}

- (GLuint)textureHandle
{
  return textureHandle;
}

- (NSString *)description
{
  return [NSString stringWithFormat:@"<%@ %x, thread %x>:\n material %@\n, texture handle %d", [self class], self, [NSThread currentThread], [material description],textureHandle];
}

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone
{
  G3DState *copy = [super copyWithZone:zone];

  copy->material = [material copy];

  [copy setTextureHandle:textureHandle];
  [copy setIsTranslucent:isTranslucent];
  [copy setAlphaClamp:alphaClamp];
  [copy setAlphaFunc:alphaFunc];
  [copy setShadeMode:shadeMode];
  [copy setEnable:enable];
  copy->ignore = ignore;

  return copy;
}

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder
{
  [super encodeWithCoder:aCoder];

  [aCoder encodeObject:material];

  [aCoder encodeValueOfObjCType:@encode(BOOL) at:&isTranslucent];
  [aCoder encodeValueOfObjCType:@encode(int) at:&ignore];
  [aCoder encodeValueOfObjCType:@encode(int) at:&enable];
  [aCoder encodeValueOfObjCType:@encode(int) at:&shadeMode];
  [aCoder encodeValueOfObjCType:@encode(int) at:&alphaFunc];
  [aCoder encodeValueOfObjCType:@encode(int) at:&textureHandle];
}

- (id)initWithCoder:(NSCoder *)aCoder
{
  self = [super initWithCoder:aCoder];

  material = [[aCoder decodeObject] retain];

  [aCoder decodeValueOfObjCType:@encode(BOOL) at:&isTranslucent];
  [aCoder decodeValueOfObjCType:@encode(int) at:&ignore];
  [aCoder decodeValueOfObjCType:@encode(int) at:&enable];
  [aCoder decodeValueOfObjCType:@encode(int) at:&shadeMode];
  [aCoder decodeValueOfObjCType:@encode(int) at:&alphaFunc];
  [aCoder decodeValueOfObjCType:@encode(int) at:&textureHandle];
  return self;
}

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver
{
  [super encodeWithSceneArchiver:archiver];

  [archiver encodeObject:material named:@"material"];

  [archiver encodeBOOL:isTranslucent named:@"isTranslucent"];
  [archiver encodeInt:ignore named:@"ignore"];
  [archiver encodeInt:enable named:@"enable"];
  [archiver encodeInt:shadeMode named:@"shadeMode"];
  [archiver encodeInt:alphaFunc named:@"alphaFunc"];
  [archiver encodeUnsignedInt:textureHandle named:@"textureHandle"];
}

- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver
{
  self = [super initWithSceneUnarchiver:unarchiver];

  material = [[unarchiver decodeObjectNamed:@"material"] retain];

  [unarchiver decodeBOOL:&isTranslucent named:@"isTranslucent"];
  [unarchiver decodeInt:&ignore named:@"ignore"];
  [unarchiver decodeInt:&enable named:@"enable"];
  [unarchiver decodeInt:&shadeMode named:@"shadeMode"];
  [unarchiver decodeInt:&alphaFunc named:@"alphaFunc"];
  [unarchiver decodeUnsignedInt:&textureHandle named:@"textureHandle"];

  return self;
}

@end






