/* 
 * G3DGLXCamera.m created by probert on 2001-07-20 20:42:55 +0000
 *
 * Project GLXKit
 *
 * Created with ProjectCenter - http://www.projectcenter.ch
 *
 * $Id: G3DGLXCamera.m,v 1.5 2001/07/22 11:58:25 probert Exp $
 */

#import "G3DGLXCamera.h"

const long X11WindowDefaultInput = KeyPressMask | KeyReleaseMask |
                                   ButtonPressMask | ButtonReleaseMask |
                                   PointerMotionMask |
                                   ExposureMask | StructureNotifyMask;

@implementation G3DGLXCamera

//-----------------------------------------------------------------------------
// Init methods
//-----------------------------------------------------------------------------

- (id)initWithSize:(NSSize)aSize
{
    return [self initWithSize:aSize input:X11WindowDefaultInput source:0 
                       display:0];
}

- (id)initWithSize:(NSSize)aSize input:(long)mask source:(int)src 
           display:(Display *)dpy
{
    if( self = [super initWithSize:aSize] )
    {
	XEvent event;
        unsigned long b;
        int attrib[] = { GLX_RGBA,
                         GLX_RED_SIZE, 1,
                         GLX_GREEN_SIZE, 1,
			 GLX_BLUE_SIZE, 1,
			 GLX_DOUBLEBUFFER,
			 None };
	
	ctx        = 0;
	vinfo      = 0;

        flags      = 0;
        window     = 0;
        screen     = 0;
        nullCursor = (Cursor)-1;

        if( dpy )
        {
            display = dpy;
        }
        else
        {
	    display = XOpenDisplay( NULL );
	    if( ! display )
	    {
		error = [NSString stringWithCString:"Can't open X display"];
	    }
            flags |= kOpenedDisplay;
        }

	if( src )
	{
	    screen = src;
	}
	else
	{
	    screen = DefaultScreen( display );
	}

	b = BlackPixel( display, screen );
	window = XCreateSimpleWindow( display, DefaultRootWindow( display ), 0,
	                              0, (int)_size.width, (int)_size.height, 0,
				      b, b );
        
	wmDeleteAtom = XInternAtom( display, "WM_DELETE_WINDOW", False );
	XSetWMProtocols( display, window, &wmDeleteAtom, 1 );

	XSelectInput( display, window, mask );

	// GLX initialisation

	if( glXQueryExtension( display, 0, 0 ) == 0 )
	{
	    [self autorelease];
	    self = nil;

	    return nil;
	}

	vinfo = glXChooseVisual( display, screen, attrib );
	if( !vinfo )
	{
	    [self autorelease];
	    self = nil;

	    return nil;
	}

	ctx = glXCreateContext( display, vinfo, NULL, True );
	if( !ctx )
	{
	    [self autorelease];
	    self = nil;

	    return nil;
	}

        XMapWindow( display, window);
	if( !glXMakeCurrent( display, window, ctx ))
	{
	    [self autorelease];
	    self = nil;

	    return nil;
	}

	while (1)
	{
	    XNextEvent(display, &event);

	    if (event.type == Expose)
		break;
	}

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glXSwapBuffers( display, window );
    }

    return self;
}

- (void)dealloc
{
    if( display )
    {
        if( window )
	{
	    if( nullCursor != (Cursor) - 1 )
	    {
	        XFreeCursor( display, nullCursor );
	    }
	    XDestroyWindow( display, window );

	    if( flags & kOpenedDisplay )
	    {
	        XCloseDisplay( display );
	    }
	}

	if( ctx )
	{
	    glXDestroyContext( display, ctx );
	}
    }

    [super dealloc];
}

//-----------------------------------------------------------------------------
// Render  method
//-----------------------------------------------------------------------------

- (void)render
{
    glXMakeCurrent( display, window, ctx );

    [super render];

    glXSwapBuffers( display, window );
}

//-----------------------------------------------------------------------------
// X11  methods
//-----------------------------------------------------------------------------

- (Display *)display
{
    return display;
}

- (int)screen;
{
    return screen;
}

- (Window)window;
{
    return window;
}

- (int)displayWidth;
{
    return XDisplayWidth( display, screen );
}

- (int)displayHeight;
{
    return XDisplayHeight( display, screen );
}

- (void)setFrame:(NSRect)aFrame display:(BOOL)yn
{
    NSSize newSize = aFrame.size;

    [self setViewportWithSize:newSize];
    XMoveResizeWindow( display, window, aFrame.origin.x, aFrame.origin.y, 
                       newSize.width, newSize.height );

    if( yn )
    {
        [self render];
    }
}

- (NSString *)title;
{
    return title;
}

- (void)setTitle:(NSString *)aTitle;
{
    [title autorelease];
    title = [aTitle copy];

    XStoreName( display, window, [title cString]);
}


- (NSString *)miniwindowTitle;
{
    return icnName;
}

- (void)setMiniwindowTitle:(NSString *)aTitle;
{
    [icnName autorelease];
    icnName = [aTitle copy];

    XSetIconName( display, window, [icnName cString] );
}

- (void)raise;
{
    XRaiseWindow( display, window );
}

- (void)show;
{
    XMapWindow( display, window );
}

- (void)hide;
{
    XUnmapWindow( display, window );
}

- (BOOL)isVisible;
{
    XWindowAttributes wa;

    XGetWindowAttributes( display, window, &wa );

    return (wa.map_state == IsViewable);
}


- (void)iconify;
{
    XIconifyWindow( display, window, screen );
}

- (void)unIconify;
{
    XMapWindow( display, window );
}


- (void)flushWindow;
{
    XFlush( display );
}

- (void)syncWindow:(BOOL)discard;
{
    XSync( display, discard );
}

- (Cursor)createNullCursor
{
    Pixmap cursormask;
    XGCValues xgc;
    GC gc;
    XColor dummycolour;
    Cursor cursor;

    cursormask = XCreatePixmap( display, window, 1, 1, 1 );

    xgc.function = GXclear;
    gc = XCreateGC( display, cursormask, GCFunction, &xgc );
    XFillRectangle( display, cursormask, gc, 0, 0, 1, 1 );
    
    dummycolour.pixel = 0;
    dummycolour.red   = 0;
    dummycolour.flags = 04;
    cursor = XCreatePixmapCursor( display, cursormask, cursormask, &dummycolour,
                                  &dummycolour, 0, 0 );

    XFreePixmap( display, cursormask );
    XFreeGC( display, gc );

    return cursor;
}

- (void)hideCursor;
{
    if( nullCursor == (Cursor) -1 )
    {
	nullCursor = [self createNullCursor];
    }

    XDefineCursor( display, window, nullCursor );
}

- (void)showCursor;
{
    XUndefineCursor( display, window );
}

//-----------------------------------------------------------------------------
// GLX methods
//-----------------------------------------------------------------------------

- (GLXContext)context
{
    return ctx;
}

- (XVisualInfo*)visual;
{
    return vinfo;
}


- (void)makeCurrent;
{
    glXMakeCurrent( display, window, ctx );
}

- (void)swapBuffers;
{
    glXSwapBuffers( display, window );
}

//-----------------------------------------------------------------------------
// Dummy implementations - need subclassing!
//-----------------------------------------------------------------------------

- (void)unknownEvent:(XEvent*)event;
{
}

- (void)deleteEvent:(XEvent*)event;
{
}

- (void)configureEvent:(XConfigureEvent*)event;
{
}

- (void)buttonDown:(XButtonEvent*)event;
{
}

- (void)buttonUp:(XButtonEvent*)event;
{
}

- (void)motionEvent:(XMotionEvent*)event;
{
}

- (void)keyDown:(XKeyEvent*)event;
{
}

- (void)keyUp:(XKeyEvent*)event;
{
}

- (void)focusIn:(XFocusChangeEvent*)event;
{
}

- (void)focusOut:(XFocusChangeEvent*)event;
{
}

- (void)exposeEvent:(XExposeEvent*)event;
{
}


//-----------------------------------------------------------------------------
// X11 error stuff
//-----------------------------------------------------------------------------

- (NSString *)errorDescription;
{
    return error;
}


@end
